#! /usr/bin/env ruby

require 'optionparser'
require 'shellwords'
require 'pathname'

FUSE = true

MOUNT_OVERLAY = FUSE ? %w(fuse-overlayfs) : %w(sudo mount -t overlay)
UMOUNT = FUSE ? %w(fusermount -u) : %w(sudo umount)

def cmd *args
  $stderr.puts "+ #{args.shelljoin}"
  system *args
  $?.success?
end

def umount dir
  cmd('mountpoint', '-q', dir) && (
    cmd(*UMOUNT, dir) || fail('Umount failed'))
end

Dir.chdir __dir__
base = 'ovrlay'

dirs = ['base']
misc = Dir.glob('*', base: File.join(base, 'mod/misc')).to_set

save = true
OptionParser.new do |opt|
  opt.on('--no-save') { save = false }
end.parse!

ARGV.each do |a|
  if misc.member? a
    dirs.unshift File.join('mod/misc', a)
  else
    fail "Unknown mod #{a}"
  end
end

dirs.unshift 'save' if save
fail if dirs.size < 2

dirs.map! {|d| File.join(base, d).gsub %r{([\\,:])}, '\\\1' }
work = File.join base, 'idiotism'
# note: a single mount option has an undocumented length limit, enough to reach
# by 3-4 mods... lowerdir+ works, but has weird syntax
ld = FUSE ? "lowerdir=#{dirs[1...].join ':'}," :
       dirs[1...].map {|x| "lowerdir+=#{x}," }.join
opts = "#{ld}upperdir=#{dirs[0]},workdir=#{work},timeout=1"
tmp_mnt = File.join(base, 'mount')
real_mnt = 'mount'

umount real_mnt
umount tmp_mnt

cmd(*MOUNT_OVERLAY, '-o', opts, 'nothing', tmp_mnt) ||
  fail('Tmp mount failed')

cmd('mountpoint', '-q', real_mnt) ||
  cmd('ciopfs', tmp_mnt, real_mnt) ||
  fail('Real mout failed')
